"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.safeReadPackageFromDir = exports.safeReadPackage = exports.fromDir = void 0;
const error_1 = __importDefault(require("@pnpm/error"));
const util_1 = require("util");
const path = require("path");
const readPackageManifestCB = require("read-package-json");
const readPackageManifest = util_1.promisify(readPackageManifestCB);
async function readPkg(pkgPath) {
    try {
        return await readPackageManifest(pkgPath);
    }
    catch (err) { // eslint-disable-line
        if (err.code)
            throw err;
        throw new error_1.default('BAD_PACKAGE_JSON', `${pkgPath}: ${err.message}`);
    }
}
exports.default = readPkg;
function fromDir(pkgPath) {
    return readPkg(path.join(pkgPath, 'package.json'));
}
exports.fromDir = fromDir;
async function safeReadPackage(pkgPath) {
    try {
        return await readPkg(pkgPath);
    }
    catch (err) {
        if (err.code !== 'ENOENT')
            throw err;
        return null;
    }
}
exports.safeReadPackage = safeReadPackage;
function safeReadPackageFromDir(pkgPath) {
    return safeReadPackage(path.join(pkgPath, 'package.json'));
}
exports.safeReadPackageFromDir = safeReadPackageFromDir;
//# sourceMappingURL=index.js.map